#include "gamut2dcommand.h"
#include "gamutmapping/gamutmapping2destimator.h"
#include "metadata/metadatastorage.h"
#include "pipeline/single_illuminant_pipeline.h"
#include <opencv2/highgui/highgui.hpp>

namespace illumestimators {

Gamut2DCommand::Gamut2DCommand():
	Command("gamut2d",
		config,
		"Michael Bleier",
		"michael.bleier@mb.stud.uni-erlangen.de"
	)
{
}

int Gamut2DCommand::execute()
{
	GamutMapping2DEstimator estimator((illumestimators::GamutMapping2DEstimator::Method) config.method, config.n, config.sigma);

	MetadataStorage storage(config.metaFiles);

	if (!SingleIlluminantPipeline::loadIlluminantEstimator(estimator, config.loadFile)) {
		if (!SingleIlluminantPipeline::runTraining(estimator, config.trainingFiles, storage, config.verbosity)) {
			std::cerr << "Error no training data and no classifier loaded. Aborted." << std::endl;
			return 1;
		}
	}
	SingleIlluminantPipeline::saveIlluminantEstimator(estimator, config.saveFile);

	Illum estimate;
	double error;

	if (config.inputFile.length() > 0) {
		if (!SingleIlluminantPipeline::runEstimator(estimate, error, estimator, config.inputFile, storage, config.verbosity)) {
			std::cout << "Gamut3DCommand: running estimator failed, aborted.\n" << std::endl;
			return -1;
		}
	}


	return 0;
}

void Gamut2DCommand::printShortHelp() const
{
	std::cout << "Gamut Mapping in 2D-chromaticity space." << std::endl;
}

void Gamut2DCommand::printHelp() const
{
	std::cout << "Gamut Mapping in 2D-chromaticity space." << std::endl;
}

} // namespace illumestimators
